import setuptools


setuptools.setup(
    name="rgbprint",
    version="3.0.5",
    license="GNU",
    author="ddjerqq",
    author_email="ddjerqq@gmail.com",
    url="https://github.com/ddjerqq/rgbprint",
    keywords="rgb print gradient color colorama",
    description="Print rgb colors and gradients in your terminal. Official github: https://github.com/ddjerqq/rgbprint",
    packages=["rgbprint"]
)
